/*
    Copyright (C) 2007    Dmitry Ivanov, vonami@gmail.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef PARTIMAGENG_ENDIAN
#define PARTIMAGENG_ENDIAN

// partimage-ng relies on libc's byteswap.h and endian.h
#include <endian.h>
#include <byteswap.h>
#include <sys/types.h>

#if (__BYTE_ORDER == __LITTLE_ENDIAN)

#define img16_cpu(x) ((u_int16_t)(x))
#define img32_cpu(x) ((u_int32_t)(x))
#define img64_cpu(x) ((u_int64_t)(x))

#define cpu_img16(x) ((u_int16_t)(x))
#define cpu_img32(x) ((u_int32_t)(x))
#define cpu_img64(x) ((u_int64_t)(x))

#elif (__BYTE_ORDER == __BIG_ENDIAN)

#define img16_cpu(x) bswap_16((u_int16_t)(x))
#define img32_cpu(x) bswap_32((u_int32_t)(x))
#define img64_cpu(x) bswap_64((u_int64_t)(x))

#define cpu_img16(x) bswap_16((u_int16_t)(x))
#define cpu_img32(x) bswap_32((u_int32_t)(x))
#define cpu_img64(x) bswap_64((u_int64_t)(x))

#else

#error "Unknown byte order"
#error "__BYTE_ORDER should be defined either __LITTLE_ENDIAN or __BIG_ENDIAN."

#endif /* __BYTE_ORDER */

#endif /* PARTIMAGENG_ENDIAN */
