/*
    Copyright (C) 2007    Dmitry Ivanov, vonami@gmail.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef PARTIMAGENG_PARTIMAGENG
#define PARTIMAGENG_PARTIMAGENG

#include <sys/types.h>

#define PARTIMAGE_NG_MAGIC                      "PARTIMAGE_NG_IMAGE_FILE"
#define PARTIMAGE_NG_DISK_IMAGE                 0
#define PARTIMAGE_NG_PARTITION_IMAGE            1

namespace PartimageNG {

        //  should be 1024 byte long
        struct ImageHeader {
                char magic[32];                 // magic string "PARTIMAGE-NG IMAGE FILE"
                u_int16_t version;              // image file version
                u_int8_t type;                  // image file type: the whole disk or one partition
                u_int16_t partitionsCount;      // number of partitions stored in the image file
                u_int16_t descriptionSize;      // length of the description (in bytes)
                u_int16_t labelnameSize;        // length of the label name (in bytes)
                u_int16_t rawdataCount;         // number of the saved raw data blocks
                u_int8_t reserved[981];         // reserved for future use
        } __attribute__((__packed__));

        // should be 64 byte long
        struct RawdataHeader {
                u_int64_t offset;               // offset from the beginnging of the disk
                u_int64_t size;                 // size of the saved data
                u_int8_t reserved[48];          // reserved for future use
        } __attribute__((__packed__));

        //  should be 1024 byte long
        struct PartitionHeader {
                u_int16_t type;                 // partition type: primary, extended, ...
                u_int64_t startSector;          // start sector
                u_int64_t endSector;            // end sector
                u_int64_t blockSize;            // size of the filesystem block
                u_int64_t blocksCount;          // total number of blocks on the filesystem
                u_int64_t usedBlocks;           // number of used blocks on the filesystem
                u_int16_t fsnameSize;           // length of the filesystem name (in bytes)
                u_int64_t bitmapSize;           // size of the bitmap (in bytes)
                u_int64_t bitmapOffset;         // offset of the bitmap (in bits)
                char reserved[964];             // reserved for future use
        } __attribute__((__packed__));

        struct Bitmap {
                u_int64_t offset;
                u_int64_t count;
                u_int64_t allocsize;
                u_int8_t *bitmap;
        };

        enum IOErrorsHandling {
                IgnoreIOErrors,
                StopIOErrors
        };

        ssize_t readn(int fd, const void *buffer, size_t size, IOErrorsHandling errHandling = StopIOErrors);
        ssize_t writen(int fd, const void *buffer, size_t size, IOErrorsHandling errHandling = StopIOErrors);

        ssize_t preadn(int fd, const void *buffer, size_t size, off_t offset, IOErrorsHandling errHandling = StopIOErrors);
        ssize_t pwriten(int fd, const void *buffer, size_t size, off_t offset, IOErrorsHandling errHandling = StopIOErrors);

        bool test_bit(const struct Bitmap *bitmap, u_int64_t bit);
        void set_bit(struct Bitmap *bm, u_int64_t bit, int value);

        int convertToUTF8(char **out, size_t *outsize, const char *in, size_t insize, const char *incodeset);
        int convertFromUTF8(char **out, size_t *outsize, const char *in, size_t insize, const char *outcodeset);
}

#endif /* PARTIMAGENG_PARTIMAGENG */
