/*
    Copyright (C) 2007    Dmitry Ivanov, vonami@gmail.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef PARTIMAGENG_PARTITIONMANAGER
#define PARTIMAGENG_PARTITIONMANAGER

#include <list>
#include <map>
#include <string>
#include <sys/types.h>

extern "C" {
        #include <parted/constraint.h>
        #include <parted/device.h>
        #include <parted/disk.h>
}

#include "partition.h"

namespace PartimageNG {
        class PartitionManager;
}

using namespace std;

class PartimageNG::PartitionManager {

public:

        enum InstantiateOption {
                DetectFilesystem,
                ForceUnknownFilesystem
        };

public:

        static int createLabel(const char *diskpath, const char *labelname);
        static const char* createPartition(const char *diskpath, u_int16_t type, const char *fsname,
                                           u_int64_t startSector, u_int64_t endSector);
        static bool formatPartition(const char *partpath, const char *fsname);
        static const char* labelname(const char *diskpath);
        static const char* filesystem(const char *partpath);
        static list<string> partitionsList(const char *diskpath);
        static Partition* instantiatePartition(const char *partpath, InstantiateOption opt = DetectFilesystem);
        static bool isDrive(const char *drivepath);
        static bool isPartition(const char *partpath);
        static bool isBusy(const char *devicepath);

private:

        static void probe();
        static void dump();

private:

        static bool probed;
        static map<string, list<string> > allDisks;
        static map<string, string> allLabels;
        static map<string, PedPartition *> allPartitions;
};

#endif /* PARTIMAGENG_PARTITIONMANAGER */
