/*
    Copyright (C) 2007    Dmitry Ivanov, vonami@gmail.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <cstdlib>
#include <iostream>

#include "unknownpartition.h"

using namespace std;
using namespace PartimageNG;

UnknownPartition::UnknownPartition(PedPartition *pedPartition)
        : Partition(pedPartition)
{
        if ( type() != PED_PARTITION_EXTENDED ) {
                m_bitmap.allocsize = blocksCount() >> 3;
                if ( (blocksCount() & 0x07) != 0 ) {
                        m_bitmap.allocsize++;
                }

                m_bitmap.bitmap = (u_int8_t *) malloc(m_bitmap.allocsize);
                memset(m_bitmap.bitmap, 0xFF, m_bitmap.allocsize);
                m_bitmap.count = blocksCount();
                m_bitmap.offset = 0;

                cerr << "UnknownPartition info:" << endl;
                cerr << "Block size = " << 512 << endl;
                cerr << "Blocks count = " << m_bitmap.count << endl;
                cerr << "Used blocks count = " << m_bitmap.count << endl;
                cerr << "Bitmap allocated size = " << m_bitmap.allocsize << endl;
        }
        else {
                m_bitmap.allocsize = 0;
        }
}

UnknownPartition::~UnknownPartition()
{
        if ( m_bitmap.allocsize != 0)
                free(m_bitmap.bitmap);
}

size_t UnknownPartition::blockSize() const
{
        return 512;
}

u_int64_t UnknownPartition::blocksCount() const
{
        return endSector() - startSector() + 1;
}

u_int64_t UnknownPartition::usedBlocks() const
{
        return endSector() - startSector() + 1;
}

const struct Bitmap* UnknownPartition::bitmap()
{
        return &m_bitmap;
}

bool UnknownPartition::dumpInternalBitmap(int dumpd) const
{
        if ( writen(dumpd, m_bitmap.bitmap, m_bitmap.allocsize) == -1 ) {
                perror("Writing the bitmap failed");
                return false;
        }

        return true;
}
